@@ -5,7 +5,7 @@ class Viewbox():
     """
     def convert_srs(self, new_wkid):
         """Return a new Viewbox object with the specified SRS."""
-        return self # not yet implemented
+        return self  # not yet implemented
 
     def __init__(self, left=-180, top=90, right=180, bottom=-90, wkid=4326):
         """
@@ -16,14 +16,15 @@ class Viewbox():
         :arg wkid: Well-known ID for spatial reference system (default ``4326``)
         """
         bounds = left, right, bottom, top
-        if not all([isinstance(x, (int, long, float)) for x in bounds]):
+        if not all([isinstance(x, (int, float)) for x in bounds]):
             raise ValueError('One or more bounds (%s) is not a real number.' % bounds)
         if left > right:
             raise ValueError('Left x-coord must be less than right x-coord.')
         if bottom > top:
             raise ValueError('Bottom y-coord must be less than top y-coord.')
-        for k in locals().keys():
-            if k != 'self': setattr(self, k, locals()[k])
+        for k in list(locals().keys()):
+            if k != 'self':
+                setattr(self, k, locals()[k])
 
     def to_bing_str(self):
         """
@@ -33,6 +34,24 @@ class Viewbox():
         vb = self.convert_srs(4326)
         return '%s,%s,%s,%s' % (vb.bottom, vb.left, vb.top, vb.right)
 
+    def to_pelias_dict(self):
+        """
+        Convert Viewbox object to a string that can be used by Pelias
+        as a query parameter.
+        """
+        vb = self.convert_srs(4326)
+        return {
+            'boundary.rect.min_lat': vb.bottom,
+            'boundary.rect.min_lon': vb.left,
+            'boundary.rect.max_lat': vb.top,
+            'boundary.rect.max_lon': vb.right
+        }
+
+    def to_google_str(self):
+        """ Convert to Google's bounds format: 'latMin,lonMin|latMax,lonMax' """
+        vb = self.convert_srs(4326)
+        return '%s,%s|%s,%s' % (vb.bottom, vb.left, vb.top, vb.right)
+
     def to_mapquest_str(self):
         """
         Convert Viewbox object to a string that can be used by
@@ -62,6 +81,7 @@ class Viewbox():
             raise Exception('One or more values could not be cast to a number. '
                             'Four bounding points must be real numbers. '
                             'WKID must be an integer.')
+
     def __repr__(self):
         top = "y=%s" % self.top
         right = "x=%s" % self.right
@@ -75,20 +95,19 @@ class Viewbox():
                 return str_[:MAX_CHARS]
             if align == 'L':
                 return str_
-            num_spaces = (MAX_CHARS - str_len) # num spaces to pad right-aligned
+            num_spaces = (MAX_CHARS - str_len)  # num spaces to pad right-aligned
             if align == 'C':
                 num_spaces = int(num_spaces / 2)
             padding = num_spaces * ' '
             return '%s%s' % (padding, str_)
 
-        return    '          %s\n'\
-                  '        ------------\n'\
-                  '        |          |\n'\
-                  '%s|          |%s\n'\
-                  '        |          |\n'\
-                  '        ------------\n'\
-                  '          %s' % (lbl(top, 'C'), lbl(left, 'R'), lbl(right, 'L'), lbl(bottom, 'C'))
-
+        return '          %s\n'\
+               '        ------------\n'\
+               '        |          |\n'\
+               '%s|          |%s\n'\
+               '        |          |\n'\
+               '        ------------\n'\
+               '          %s' % (lbl(top, 'C'), lbl(left, 'R'), lbl(right, 'L'), lbl(bottom, 'C'))
 
 
 class PlaceQuery():
@@ -131,8 +150,9 @@ class PlaceQuery():
                           would be returned as "Vereinigte Staaten Von Amerika"
                           instead of "United States".
     """
-        for k in locals().keys():
-            if k not in ['self', 'kwargs']: setattr(self, k, locals()[k])
+        for k in list(locals().keys()):
+            if k not in ['self', 'kwargs']:
+                setattr(self, k, locals()[k])
         if query == '' and address == '' and city == '' and state == '' and postal == '':
             raise Exception('Must provide query or one or more of address, city, state, and postal.')
         for k in kwargs:
@@ -154,7 +174,7 @@ class Candidate():
     """
 
     def __init__(self, locator='', score=0, match_addr='', x=None, y=None,
-        wkid=4326, **kwargs):
+                 wkid=4326, **kwargs):
         """
         :arg locator: Locator used for geocoding (default ``''``)
 
@@ -186,8 +206,9 @@ class Candidate():
         return these values.
         """
 
-        for k in locals().keys():
-            if k not in ['self', 'kwargs']: setattr(self, k, locals()[k])
+        for k in list(locals().keys()):
+            if k not in ['self', 'kwargs']:
+                setattr(self, k, locals()[k])
         for k in kwargs:
             setattr(self, k, kwargs[k])
 
