@@ -1,7 +1,8 @@
+# flake8: noqa: F401
 from .bing import Bing
-from .esri import EsriWGS, EsriWGSSSL
+from .esri import EsriWGS
 from .us_census import USCensus
 from .mapquest import MapQuest, MapQuestSSL
 from .nominatim import Nominatim
-from .mapzen import Mapzen
+from .pelias import Pelias
 from .google import Google
