@@ -10,11 +10,11 @@ from omgeo.preprocessors import (CancelIfPOBox, CancelIfRegexInAttr, CountryPreP
                                  RequireCountry, ParseSingleLine, ReplaceRangeWithNumber)
 from omgeo.postprocessors import (AttrFilter, AttrExclude, AttrRename,
                                   AttrSorter, AttrReverseSorter, UseHighScoreIfAtLeast,
-                                  GroupBy, ScoreSorter, SnapPoints)
+                                  GroupBy, ScoreSorter, SnapPoints, AttrListIncludes, AttrListExcludes)
 
 BING_MAPS_API_KEY = os.getenv("BING_MAPS_API_KEY")
 MAPQUEST_API_KEY = os.getenv("MAPQUEST_API_KEY")
-MAPZEN_API_KEY = os.getenv("MAPZEN_API_KEY")
+PELIAS_API_KEY = os.getenv("PELIAS_API_KEY")
 GOOGLE_API_KEY = os.getenv("GOOGLE_API_KEY")
 ESRI_CLIENT_ID = os.getenv("ESRI_CLIENT_ID")
 ESRI_CLIENT_SECRET = os.getenv("ESRI_CLIENT_SECRET")
@@ -44,9 +44,8 @@ class GeocoderTest(OmgeoTestCase):
     BING_KEY_REQUIRED_MSG = 'Enter a Bing Maps API key to run the Bing tests.'
     MAPQUEST_KEY_REQUIRED_MSG = 'Enter a MapQuest API key to run the MapQuest tests. '\
                                 'Keys can be obtained at http://developer.mapquest.com/.'
-    MAPZEN_KEY_REQUIRED_MSG = 'Enter a Mapzen Search API key to run Mapzen ' \
-                              'tests. Keys can be obtained at ' \
-                              'https://mapzen.com/developers/sign_in.'
+    PELIAS_KEY_REQUIRED_MSG = 'Enter a Pelias service search API key to run Pelias tests. '\
+                              'Keys can be obtained at https://geocode.earth/.'
     GOOGLE_KEY_REQUIRED_MSG = 'Enter a Google API key to run Google tests.'
     ESRI_KEY_REQUIRED_MSG = 'Enter a Esri Client ID & Secret to run authenticated Esri tests.'
 
@@ -79,6 +78,7 @@ class GeocoderTest(OmgeoTestCase):
             'quebec_hyphenated': PlaceQuery('227-227A Rue Commerciale, Saint-Louis-du-Ha! Ha! QC'),
             'senado_mx': PlaceQuery('Paseo de la Reforma 135, Tabacalera, Cuauhtémoc, Distrito Federal, 06030'),
             'senado_mx_struct': PlaceQuery(address='Paseo de la Reforma 135', neighborhood='Tabacalera, Cuauhtémoc', subregion='', state='Distrito Federal', postal='06030', country='MX'),
+            'robert_cheetham': PlaceQuery('Robert Cheetham, Philadelphia'),
             # European Addresses:
             'london_pieces': PlaceQuery(address='31 Maiden Lane', city='London', country='UK'),
             'london_one_line': PlaceQuery('31 Maiden Lane, London WC2E', country='UK'),
@@ -100,13 +100,17 @@ class GeocoderTest(OmgeoTestCase):
             self.g_mapquest_ssl = Geocoder([['omgeo.services.MapQuestSSL',
                                            {'settings': mapquest_settings}]])
 
-        if MAPZEN_API_KEY is not None:
-            mapzen_settings = dict(api_key=MAPZEN_API_KEY)
-            self.g_mapzen = Geocoder([['omgeo.services.Mapzen', {'settings': mapzen_settings}]])
+        if PELIAS_API_KEY is not None:
+            pelias_settings = dict(api_key=PELIAS_API_KEY)
+            self.g_pelias = Geocoder([['omgeo.services.Pelias', {'settings': pelias_settings}]])
 
         if GOOGLE_API_KEY is not None:
             self.g_google = Geocoder([['omgeo.services.Google',
                                      {'settings': {'api_key': GOOGLE_API_KEY}}]])
+            self.g_google_wo_postprocess = Geocoder(
+                [['omgeo.services.Google',
+                  {'settings': {'api_key': GOOGLE_API_KEY}, 'postprocessors': []}]]
+            )
 
         #: main geocoder used for tests, using default APIs
         self.g = Geocoder()
@@ -124,7 +128,7 @@ class GeocoderTest(OmgeoTestCase):
         if MAPQUEST_API_KEY is not None:  # MapQuest's open Nominatime API now also requires a key
             self.g_nom = Geocoder([['omgeo.services.Nominatim', {}]])
 
-        self.g_census = Geocoder([['omgeo.services.USCensus', {}]])
+        self.g_census = Geocoder([['omgeo.services.USCensus', {'settings': {'timeout': 30}}]])
 
         ESRI_WGS_LOCATOR_MAP = {'PointAddress': 'rooftop',
                                 'StreetAddress': 'interpolation',
@@ -144,7 +148,8 @@ class GeocoderTest(OmgeoTestCase):
         self.g_esri_wgs_postal_ok = Geocoder(GEOCODERS_POSTAL_OK)
 
         #: geocoder with fast timeout
-        self.impatient_geocoder = Geocoder([['omgeo.services.EsriWGS', {'settings': {'timeout': 0.01}}]])
+        self.impatient_geocoder = Geocoder(
+            [['omgeo.services.EsriWGS', {'settings': {'timeout': 0.001}}]])
 
     def tearDown(self):
         pass
@@ -219,6 +224,16 @@ class GeocoderTest(OmgeoTestCase):
         candidates = self.g_esri_wgs_auth.get_candidates(self.pq['azavea'])
         self.assertOneCandidate(candidates)
 
+    def test_esri_short_region(self):
+        """Ensure that Esri uses region abbreviations"""
+        candidate = self.g_esri_wgs.get_candidates(self.pq["azavea"])[0]
+        self.assertEqual(candidate.match_region, "PA")
+
+    def test_google_short_region(self):
+        """Ensure that Google uses region abbreviations"""
+        candidate = self.g_google.get_candidates(self.pq["azavea"])[0]
+        self.assertEqual(candidate.match_region, "PA")
+
     @unittest.skipIf(BING_MAPS_API_KEY is None, BING_KEY_REQUIRED_MSG)
     def test_geocode_bing(self):
         """Test Azavea's address using Bing geocoder"""
@@ -237,10 +252,10 @@ class GeocoderTest(OmgeoTestCase):
         candidates = self.g_mapquest_ssl.get_candidates(self.pq['azavea'])
         self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
 
-    @unittest.skipIf(MAPZEN_API_KEY is None, MAPZEN_KEY_REQUIRED_MSG)
-    def test_geocode_mapzen(self):
-        """Test Azavea's address using Mapzen geocoder"""
-        candidates = self.g_mapzen.get_candidates(self.pq['azavea'])
+    @unittest.skipIf(PELIAS_API_KEY is None, PELIAS_KEY_REQUIRED_MSG)
+    def test_geocode_pelias(self):
+        """Test Azavea's address using Pelias geocoder"""
+        candidates = self.g_pelias.get_candidates(self.pq['azavea'])
         self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
 
     @unittest.skip("FIXME")
@@ -286,14 +301,15 @@ class GeocoderTest(OmgeoTestCase):
         """
         candidates = self.g_bing.get_candidates(self.pq['karori'])
         self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
-        self.assertEqual(any([('102' in c.match_addr and '6012' in c.match_addr) for c in candidates]),
+        self.assertEqual(
+            any([('102' in c.match_addr and '6012' in c.match_addr) for c in candidates]),
             True, 'Could not find bldg. no. "102" and postcode "6012" in any address.')
 
     def _test_address_components(self, candidate):
-            for field in ['match_streetaddr', 'match_city', 'match_subregion', 'match_region',
-                          'match_postal', 'match_country']:
-                self.assertIsNotNone(getattr(candidate, field, None),
-                                     msg='Missing address component %s' % field)
+        for field in ['match_streetaddr', 'match_city', 'match_subregion', 'match_region',
+                      'match_postal', 'match_country']:
+            self.assertIsNotNone(getattr(candidate, field, None),
+                                 msg='Missing address component %s' % field)
 
     def _test_geocode_results_all_(self, verbosity=0, geocoder=Geocoder(),
                                    expected_results=16):
@@ -314,11 +330,12 @@ class GeocoderTest(OmgeoTestCase):
             else:
                 queries_with_results += 1
                 logger.info('Input:  %s' % self.pq[place].query)
-                logger.info(map(lambda c: 'Output: %r (%s %s)\n' %
-                                (c.match_addr,
-                                 c.geoservice,
-                                 [c.locator, c.score, c.confidence, c.entity]),
-                                candidates))
+                logger.info([
+                    'Output: %r (%s %s)\n' % (
+                        c.match_addr,
+                        c.geoservice,
+                        [c.locator, c.score, c.confidence, c.entity])
+                    for c in candidates])
         self.assertEqual(expected_results, queries_with_results,
                          'Got results for %d of %d queries.' % (queries_with_results, len(self.pq)))
 
@@ -343,13 +360,23 @@ class GeocoderTest(OmgeoTestCase):
 
     @unittest.skipIf(GOOGLE_API_KEY is None, GOOGLE_KEY_REQUIRED_MSG)
     def test_google_country_filter(self):
-        candidates = self.g_google.get_candidates('York')
+        candidates = self.g_google.get_candidates(PlaceQuery('York', country='US'))
         self.assertOneCandidate(candidates)
         self.assertEqual(candidates[0].match_region, 'PA')
-        candidates = self.g_google.get_candidates(PlaceQuery('York', country='UK'))
+        candidates = self.g_google_wo_postprocess.get_candidates(PlaceQuery('York', country='GB'))
         self.assertOneCandidate(candidates)
         self.assertEqual(candidates[0].match_country, 'GB')
 
+    @unittest.skipIf(GOOGLE_API_KEY is None, GOOGLE_KEY_REQUIRED_MSG)
+    def test_google_geocode_without_postprocessor_allows_people(self):
+        candidates = self.g_google_wo_postprocess.get_candidates(self.pq['robert_cheetham'])
+        self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
+
+    @unittest.skipIf(GOOGLE_API_KEY is None, GOOGLE_KEY_REQUIRED_MSG)
+    def test_google_geocode_excludes_people(self):
+        candidates = self.g_google.get_candidates(self.pq['robert_cheetham'])
+        self.assertEqual(len(candidates) == 0, True, 'Candidate(s) unexpectedly returned.')
+
 
 class GeocoderProcessorTest(OmgeoTestCase):
     """Tests using various pre- and post-processors."""
@@ -376,6 +403,10 @@ class GeocoderProcessorTest(OmgeoTestCase):
                                      score=99.9, x=-75.163, y=39.959)  # same y
         self.reading_term = Candidate(match_addr='1200 Arch St', locator='rooftop',
                                       score=99.9, x=-75.163, y=39.953)  # same x
+        self.with_address_types = Candidate(match_addr='123 Any St', locator='address',
+                                            entity_types=['address', 'place'])
+        self.with_nonsense_types = Candidate(match_addr='123 Any St', locator='address',
+                                             entity_types=['house', 'building'])
 
         self.locators_worse_to_better = ['address', 'parcel', 'rooftop']
 
@@ -413,6 +444,31 @@ class GeocoderProcessorTest(OmgeoTestCase):
         place_exp = place_in  # we should still have it because PO BOX does not match exactly
         self.assertEqual_(place_out, place_exp)
 
+    def test_pro_CancelIfRegexInAttr_all_unicode(self):
+        """Test CancelIfRegexInAttr preprocessor using unicode query, regex, and attrs."""
+        place_in = PlaceQuery(u'PO Box 123, Lindström, MN')
+        place_out = CancelIfRegexInAttr(regex=u"po box", attrs=(u'query',)).process(place_in)
+        place_exp = False
+        self.assertEqual_(place_out, place_exp)
+
+    def test_pro_CancelIfRegexInAttr_unicode_query(self):
+        """Test CancelIfRegexInAttr preprocessor using unicode query and default regex/attrs."""
+        place_in = PlaceQuery(u'PO Box 123, Lindström, MN')
+        place_out = CancelIfRegexInAttr(regex="po box", attrs=('query',)).process(place_in)
+        place_exp = False
+        self.assertEqual_(place_out, place_exp)
+
+    def test_pro_CancelIfRegexInAttr_unicode_args(self):
+        """Test CancelIfRegexInAttr preprocessor using bytestring query and unicode regex/attrs."""
+        place_in = PlaceQuery(u'PO Box 123, Lindström, MN'.encode("utf-8"))
+        # Test all 3 combinations for completeness, but only test the result once since if they
+        # fail it will be by raising exceptions
+        place_out = CancelIfRegexInAttr(regex=u"po box", attrs=('query',)).process(place_in)
+        place_out = CancelIfRegexInAttr(regex="po box", attrs=(u'query',)).process(place_in)
+        place_out = CancelIfRegexInAttr(regex=u"po box", attrs=('query',)).process(place_in)
+        place_exp = False
+        self.assertEqual_(place_out, place_exp)
+
     def test_pro_CancelIfPOBox(self):
         """Test CancelIfPOBox preprocessor."""
         place_in = PlaceQuery('PO Box 123, Philadelphia, PA')
@@ -591,6 +647,22 @@ class GeocoderProcessorTest(OmgeoTestCase):
         candidates_out = SnapPoints(distance=50).process(candidates_in)
         self.assertEqual_(candidates_out, candidates_exp)
 
+    def test_pro_filter_AttrListIncludes(self):
+        """Test AttrListIncludes postprocessor."""
+        good_values = ['address']
+        candidates_in = [self.with_address_types, self.with_nonsense_types]
+        candidates_exp = [self.with_address_types]
+        candidates_out = AttrListIncludes(good_values, 'entity_types').process(candidates_in)
+        self.assertEqual_(candidates_out, candidates_exp)
+
+    def test_pro_filter_AttrListExcludes(self):
+        """Test AttrListExcludes postprocessor."""
+        bad_values = ['house']
+        candidates_in = [self.with_address_types, self.with_nonsense_types]
+        candidates_exp = [self.with_address_types]
+        candidates_out = AttrListExcludes(bad_values, 'entity_types').process(candidates_in)
+        self.assertEqual_(candidates_out, candidates_exp)
+
 
 if __name__ == '__main__':
     logging.basicConfig(stream=sys.stdout)
